#ifndef CATEGORY_H_
#define CATEGORY_H_

#include <list>
#include <string>
#include <msp/datafile/objectloader.h>
#include "appearance.h"
#include "condition.h"
#include "rarity.h"

class FilterStatement;
class PoeFilter;
class Theme;

class Category
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Category>
	{
	private:
		const PoeFilter &poe;
		CompoundCondition *compound;
		Appearance::Loader app_loader;

	public:
		Loader(Category &, const PoeFilter &, CompoundCondition * = 0);

	protected:
		template<typename T>
		void add_range(const std::string &);

		void add_condition(Condition *);

	private:
		void and_();

		void category(const std::string &);

		template<typename T>
		void condition(typename T::Type);

		template<typename T>
		void condition_max(typename T::Type);

		template<typename T>
		void condition_min(typename T::Type);

		template<typename T>
		void condition_range(typename T::Type, typename T::Type);

		void linked_colors(const LinkedColorsCondition::Colors &);
		void or_();
	};

private:
	std::string name;
	Condition *condition;
	unsigned order;
	Appearance appearance;

public:
	Category();
	Category(const std::string &, unsigned);
	Category(const Category &);
	Category &operator=(const Category &);
	~Category();

	const std::string &get_name() const { return name; }
	unsigned get_order() const { return order; }
	const Appearance &get_appearance() const { return appearance; }
	void create_statements(std::list<FilterStatement> &) const;
};

#endif
