#ifndef CONDITION_H_
#define CONDITION_H_

#include <list>
#include <string>
#include <vector>
#include <msp/strings/format.h>
#include "rarity.h"

class CompoundCondition;
class FilterStatement;

class Condition
{
protected:
	Condition() { }
public:
	virtual ~Condition() { }

	virtual Condition *clone() const = 0;
	virtual Condition *flatten() const { return clone(); }
	virtual bool can_merge(const Condition &, const CompoundCondition &) const { return false; }
	virtual Condition *merge(const std::vector<Condition *> &, const CompoundCondition &) const { return 0; }
	virtual bool is_viable() const { return true; }
	virtual void add_lines(std::list<FilterStatement> &) const = 0;
};


class AndCondition;
class OrCondition;

class CompoundCondition: public Condition
{
protected:
	std::vector<Condition *> conditions;

public:
	virtual ~CompoundCondition();

protected:
	void clone_to(CompoundCondition &) const;

public:
	void add(Condition *);
	unsigned count() const { return conditions.size(); }
	const Condition *get(unsigned) const;

	virtual Condition *flatten() const;
protected:
	Condition *dispatch_flatten(Condition *, Condition *) const;
	Condition *merge_two(Condition *, Condition *, bool) const;
	Condition *add_merged_to(Condition *, CompoundCondition *, bool) const;
	Condition *merge_contents_to(CompoundCondition *, CompoundCondition *) const;
	virtual Condition *flatten(Condition *, Condition *) const = 0;
	virtual Condition *flatten(AndCondition *, Condition *) const = 0;
	virtual Condition *flatten(AndCondition *, AndCondition *) const = 0;
	virtual Condition *flatten(OrCondition *, Condition *) const = 0;
	virtual Condition *flatten(OrCondition *, AndCondition *) const = 0;
	virtual Condition *flatten(OrCondition *, OrCondition *) const = 0;
};


class AndCondition: public CompoundCondition
{
public:
	virtual AndCondition *clone() const;
protected:
	virtual Condition *flatten(Condition *, Condition *) const;
	virtual Condition *flatten(AndCondition *, Condition *) const;
	virtual Condition *flatten(AndCondition *, AndCondition *) const;
	virtual Condition *flatten(OrCondition *, Condition *) const;
	virtual Condition *flatten(OrCondition *, AndCondition *) const;
	virtual Condition *flatten(OrCondition *, OrCondition *) const;
public:
	virtual bool is_viable() const;
	virtual void add_lines(std::list<FilterStatement> &) const;
};


class OrCondition: public CompoundCondition
{
public:
	virtual OrCondition *clone() const;
protected:
	virtual Condition *flatten(Condition *, Condition *) const;
	virtual Condition *flatten(AndCondition *, Condition *) const;
	virtual Condition *flatten(AndCondition *, AndCondition *) const;
	virtual Condition *flatten(OrCondition *, Condition *) const;
	virtual Condition *flatten(OrCondition *, AndCondition *) const;
	virtual Condition *flatten(OrCondition *, OrCondition *) const;
public:
	virtual bool is_viable() const;
	virtual void add_lines(std::list<FilterStatement> &) const;
};


class LinkedColorsCondition: public Condition
{
public:
	struct Colors
	{
		char colors[7];
	};

private:
	Colors colors;
	
public:
	LinkedColorsCondition(const Colors &);

	virtual LinkedColorsCondition *clone() const;
	virtual void add_lines(std::list<FilterStatement> &) const;
};

void operator>>(const Msp::LexicalConverter &, LinkedColorsCondition::Colors &);

#endif
