#ifndef FILTER_H_
#define FILTER_H_

#include <list>
#include <map>
#include <string>
#include <msp/datafile/objectloader.h>
#include <msp/io/base.h>
#include "appearance.h"

class Category;
class PoeFilter;
class Theme;

class FilterStatement
{
private:
	bool show;
	std::list<std::string> lines;

public:
	FilterStatement();

	void set_show(bool);
	void add_line(const std::string &);

	static void add_line(std::list<FilterStatement> &, const std::string &);

	void write(Msp::IO::Base &) const;
};


class Filter
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Filter>
	{
	private:
		const PoeFilter &poe;

	public:
		Loader(Filter &, const PoeFilter &);

	private:
		static std::string glob_to_re(const std::string &);
		static bool category_order(const Category *, const Category *);
		void add_categories(const std::string &, bool);

		void cancel(const std::string &);
		void hide(const std::string &);
		void include(const std::string &);
		void show(const std::string &);
	};

private:
	struct Block
	{
		bool show;
		const Category *category;
		Appearance appearance;
	};

	std::list<Block> blocks;
	bool abstract;

public:
	Filter();

	bool is_abstract() const { return abstract; }
	void write(Msp::IO::Base &) const;
};

#endif
