#include <msp/core/maputils.h>
#include "theme.h"

using namespace std;
using namespace Msp;

Theme::Theme():
	base_font_size(32)
{ }

void Theme::set_color(const string &name, const Color &color)
{
	colors[name] = color;
}

const Color &Theme::get_color(const string &name) const
{
	return get_item(colors, name);
}

void Theme::set_base_font_size(unsigned size)
{
	base_font_size = size;
}

void Theme::set_appearance(const string &name, const Appearance &app)
{
	appearances[name] = app;
}

const Appearance &Theme::get_appearance(const string &name) const
{
	return get_item(appearances, name);
}

const Appearance *Theme::find_appearance(const string &name) const
{
	AppearanceMap::const_iterator i = appearances.find(name);
	return (i!=appearances.end() ? &i->second : 0);
}


Theme::Loader::Loader(Theme &t):
	DataFile::ObjectLoader<Theme>(t)
{
	add("appearance", &Loader::appearance);
	add("base_font_size", &Theme::base_font_size);
	add("color", &Loader::color);
	add("color", &Loader::color_alpha);
}

void Theme::Loader::appearance(const string &name)
{
	Appearance app;
	Appearance::Loader ldr(app, &obj);
	load_sub_with(ldr);
	obj.set_appearance(name, app);
}

void Theme::Loader::color(const string &name, unsigned r, unsigned g, unsigned b)
{
	obj.set_color(name, Color(r, g, b));
}

void Theme::Loader::color_alpha(const string &name, unsigned r, unsigned g, unsigned b, unsigned a)
{
	obj.set_color(name, Color(r, g, b, a));
}
